<?php
namespace App\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Process\Process;

/**
 * Class AbstractMultiCommand
 */
abstract class AbstractMultiCommand extends ContainerAwareCommand
{
    /**
     * @var int
     */
    private $maxProcess = 4;

    /**
     * @var Process[]
     */
    private $processes = [];

    /**
     * @var Process[]
     */
    private $runningProcesses = [];

    /**
     * Run Processes
     *
     * @param OutputInterface $output
     *
     * @return $this
     */
    protected function runProcesses(OutputInterface $output)
    {
        // Run until we have no process left
        while (count($this->processes)) {
            // Clean current running processes
            foreach ($this->runningProcesses as $pid => $p) {
                if (!$p->isRunning()) {
                    $output->writeln("End pid ".$pid);
                    
                    unset($this->runningProcesses[$pid]);
                }
            }

            // Create new running processes if needed
            while (
                $this->maxProcess > count($this->runningProcesses) &&
                count($this->processes)
                ) {
                    $p = array_shift($this->processes);
                    $output->writeln("Starting pid ".$p->getPid());
                    $p->start(function ($type, $buffer) {
                        if ('err' === $type) {
                            echo 'ERR > '.$buffer;
                        } else {
                            echo 'OUT > '.$buffer;
                        }
                    });
                    $output->writeln("cmd : ".$p->getCommandLine());
                    $this->runningProcesses[$p->getPid()] = $p;
                }
        }

        return $this;
    }

    /**
     * Set MaxProcess
     *
     * @param int $maxProcess
     *
     * @return $this
     */
    protected function setMaxProcess($maxProcess)
    {
        $this->maxProcess = $maxProcess;

        return $this;
    }

    /**
     * Add Process
     *
     * @param Process $process
     *
     * @return $this
     */
    protected function addProcess(Process $process)
    {
        $process->setTimeout(3600);
        
        $this->processes[] = $process;

        return $this;
    }
}
?>